local addon, DRTS = ...

local UnitPosition, GetPlayerFacing, GetPOITextureCoords = UnitPosition, GetPlayerFacing, GetPOITextureCoords
local pi, cos, sin, abs = math.pi, math.cos, math.sin, math.abs
local Minimap = Minimap

local MinimapRadius = {
    Inside = {
        [0] = 150.0,
        [1] = 120.0,
        [2] = 90.0,
        [3] = 60.0,
        [4] = 40.0,
        [5] = 25.0,
    },
    Outside = {
        [0] = 233.333324,
        [1] = 199.999992,
        [2] = 166.66666,
        [3] = 133.333328,
        [4] = 99.999996,
        [5] = 66.666664,
    }
}

local poisCreated = 0

local function MinimapPOI_OnEnter(self)
    GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 25, 0)
    local name = DRTS.treasures[self.id].name
    GameTooltip:SetText(name, 1, 1, 1, true)
    GameTooltip:Show()
end

local function MinimapPOI_OnLeave(self)
    GameTooltip:Hide()
end

local function Minimap_CreatePOI(index)
    local poi = CreateFrame("Button", "MinimapPOI"..index, Minimap)
    poi:SetHeight(20)
    poi:SetWidth(20)

    poi.Texture = poi:CreateTexture(poi:GetName().."Texture", "BACKGROUND")
    poi.Texture:SetTexture("Interface\\Minimap\\POIIcons")

    local x1, x2, y1, y2 = GetPOITextureCoords(197)

    poi.Texture:SetTexCoord(x1, x2, y1, y2)

    poi.Texture:SetPoint("CENTER", 0, 0)
    poi.Texture:SetHeight(12)
    poi.Texture:SetWidth(12)

    poi:SetScript("OnEnter", MinimapPOI_OnEnter)
    poi:SetScript("OnLeave", MinimapPOI_OnLeave)

    poisCreated = poisCreated + 1

    return poi
end

local function WorldPosToMinimapPos(px, py, x, y, radius)
    local locLeft, locTop, locRight, locBottom = py + radius, px + radius, py - radius, px - radius

    local h, w = abs(locRight - locLeft), abs(locBottom - locTop)
    local sx = (locTop - x) / w
    local sy = (locLeft - y) / h
    local onMap = y < locLeft and y > locRight and x < locTop and x > locBottom
    return sy, sx, onMap
end

DRTS.minimapRotationEnabled = GetCVar("rotateMinimap") ~= "0"

DRTS.UpdateMinimapIcons = function()
    local x, y, _, mapID = UnitPosition("player")
    local zoom = Minimap:GetZoom()
    local radius = DRTS.minimapOutside and MinimapRadius.Outside[zoom] or MinimapRadius.Inside[zoom]
    local mmw = Minimap:GetWidth()
    local mmh = Minimap:GetHeight()
    local pixelsperyard = mmw / (radius * 2)
    local rotation = (2 * pi) - GetPlayerFacing()

    local index = 0

    local showTreasures = DRTSOptions.enabled and DRTS.TreasureMapsIds[mapID]

    for k,v in pairs(DRTS.treasures) do
        local trCheck, tmCheck, choise1Check, choise2Check, _, _, _, _ = DRTS.GetTreasureCompletion(v)
        local distCheck = DRTS.GetDistanceToPoint(x, y, v.x, v.y) < radius

        if showTreasures and distCheck and (DRTSOptions.debug or (not DRTSOptions.hidden[k] and trCheck and tmCheck and choise1Check and choise2Check)) then
            index = index + 1
            local poi = _G["MinimapPOI"..index] or Minimap_CreatePOI(index)

            poi.id = k

            if DRTS.minimapRotationEnabled then
                local dy, dx = v.x - x, v.y - y
                local sinTheta = sin(rotation)
                local cosTheta = cos(rotation)
                local mx = ((-dx * cosTheta) - (dy * sinTheta)) * pixelsperyard
                local my = ((-dx * sinTheta) + (dy * cosTheta)) * pixelsperyard
                poi:SetPoint("CENTER", "Minimap", "CENTER", mx, my)
            else
                local mx, my = WorldPosToMinimapPos(x, y, v.x, v.y, radius)
                mx = mx * mmw
                my = -my * mmh
                poi:SetPoint("CENTER", "Minimap", "TOPLEFT", mx, my)
            end

            poi:Show()
        end
    end

    if poisCreated > index then
        for p = index + 1, poisCreated do
            local poi = _G["MinimapPOI"..p]
            poi:Hide()
            --print("hide poi: "..p)
        end
    end
    --print("Treasures in minimap range: "..index)
end
